﻿//By Vitaly Polyakov special for http://mai-tools.com

#target Illustrator
#targetengine main

function IsometryLightInterface() {
	this.windowRef = null;
	}


IsometryLightInterface.prototype.run = function(specifier)
{
	// Create a window of type palette.
	var win = new Window("palette", "Magic Isometry Free");  
	this.windowRef = win;
    
    win.specifier = specifier;
	
	win.panel = win.add("panel");
	win.panel.alignChildren = "left";
	win.angleGroup = win.panel.add("group");
	win.angleLabel = win.angleGroup.add("statictext", undefined, "Angle:");
	win.angleEdit = win.angleGroup.add("edittext", undefined, "26.57");
	win.angleEdit.characters = 7;
	
	win.btnGroup1 = win.panel.add("group");
	win.leftBtn = win.btnGroup1.add("button", undefined, "Left");
	win.rightBtn = win.btnGroup1.add("button",undefined, "Right");
	win.topBtn = win.btnGroup1.add("button", undefined, "Top");
    win.btnGroup2 = win.panel.add("group");
    win.symmetryBtn = win.btnGroup2.add("button", undefined, "Symmetry");
    win.mountBtn = win.btnGroup2.add("button", undefined, "Mount");    
    
	// Register event listeners that define the button behavior
	win.leftBtn.onClick = this.onClick;
	win.rightBtn.onClick = this.onClick;
	win.topBtn.onClick = this.onClick;
    win.symmetryBtn.onClick = this.onClick;
    win.mountBtn.onClick = this.onClick;

	win.closeBtn = win.add("button", undefined, "Close");
	win.closeBtn.alignment = "right";
	win.closeBtn.onClick = function () {
		this.window.close();
		}

	// Display the window
	win.center();
	win.show();
		
	return true;
		
}

IsometryLightInterface.prototype.onClick = function() {
	var buttonName = this.text;
	var win = this.window;
	var angle = win.angleEdit.text;
	buildMsg(buttonName, angle, win.specifier);
	
	}

function makeSelectionArray() {
    var doc = app.activeDocument;
    var items = [];
	for (var i = 0 ; i < doc.selection.length; i++) {
		items.push(doc.selection[i]);
		}
    return items;
    }

function projection(matrix) {
	
	var items = makeSelectionArray();
	for (var i = 0 ; i < items.length; i++) {
		var item = items[i];
		item.transform(matrix,true,true,true,true,1);
		}
	}

function matrixLeft(angleRad) {
	var matrix = app.getIdentityMatrix();
	matrix.mValueB = -Math.tan(angleRad);
	return matrix;
	}

function matrixRight(angleRad) {
	var matrix = app.getIdentityMatrix();
	matrix.mValueB = Math.tan(angleRad);
	return matrix;
	}

function matrixTop(angleRad) {
	var sin45 = Math.sin(45/180*Math.PI)
	var m = app.getRotationMatrix(45);
	var matrix = app.concatenateScaleMatrix(m, 100/sin45, Math.tan(angleRad)*100 / sin45);
	return matrix;
	}


function processClick (params) {
    var doc = app.activeDocument;
	if (doc.selection.length == 0) {
		alert("Select any object");
		return;
		}
	var obj = eval(params);
    var buttonName = obj.buttonName;
	var angle = Number(obj.angle);
	
	var angleRad = angle/180*Math.PI;
	var matrix;
	if (buttonName == "Left") {
		matrix = matrixLeft(angleRad);
		}
	if (buttonName == "Right") {
		matrix = matrixRight(angleRad);
		}
	if (buttonName == "Top") {
		matrix = matrixTop(angleRad);
		}
    
	if (matrix !== undefined) {
        projection(matrix);
        }
    
    if (buttonName == "Symmetry") {
        makeSymmetry(angleRad);
        }
    
    if (buttonName == "Mount") {
        makeMount(angleRad);
        }
	
}

function findOutItemIndex(items,numCoord,max) {
    var coord = items[0].geometricBounds[numCoord];
    var index = 0;
    for (var i = 1; i<items.length; i++) {
        var bounds = items[i].geometricBounds;
        if (max) {
            if (coord < bounds[numCoord]) {
                coord = bounds[numCoord];
                index = i;
                }
            }
        else {
            if (coord > bounds[numCoord]) {
                coord = bounds[numCoord];
                index = i;
                }
            }
        }
    
    return index;
    }

function makeMount(angleRad) {
   var items = makeSelectionArray();
   if ((items.length  < 2) || (items.length > 3)) {
       alert("Select 2 or 3 objects");
       return;
       }
   var topItem;
   if (items.length == 3) {
       var topItemIndex = findOutItemIndex(items,1,true,true);
       topItem = items[topItemIndex];
       var newItems = [];
       //array.splice глючит, делаем аналог
       //items = items.slice(topItemIndex,1);
       for (var i = 0; i < items.length; i++) {
           if (i!=topItemIndex) {
               newItems.push(items[i]);
               }
           }
       items = newItems;
       
   }

   var leftItem = items[findOutItemIndex(items,0,false)];
   var rightItem = items[findOutItemIndex(items,2,true)];
   
   if ((leftItem == rightItem) || (leftItem == topItem) || (rightItem == topItem)) {
       alert ("Correct positions of objects");
       return;
       }
   
   var x1rightItem = leftItem.geometricBounds[2];
   var y2rightItem = leftItem.geometricBounds[3];
   var rightDX = x1rightItem - rightItem.geometricBounds[0];
   var rightDY = y2rightItem - rightItem.geometricBounds[3];
   
   moveItem(rightItem, rightDX, rightDY);
   
   if (topItem !== undefined) {
       var x1topItem = leftItem.geometricBounds[0];
       var y2topItem = leftItem.geometricBounds[1] - leftItem.width*Math.tan(angleRad);
       var topDX = x1topItem - topItem.geometricBounds[0];
       var topDY = y2topItem - topItem.geometricBounds[3];
       
       moveItem(topItem, topDX, topDY);
       }
   }

function moveItem(item, dx , dy) {
    var matrix = app.getTranslationMatrix(dx,dy);
    item.transform(matrix,true,true,true,true,1);
    }

function makeSymmetry(angleRad){

    var matrixL = matrixLeft(angleRad);
    var matrixR = matrixRight(angleRad);
    //var matrixMirror = app.getScaleMatrix(-100,100);
    //var matrixPre = app.concatenateMatrix(matrixL,matrixMirror);
        
    var items = makeSelectionArray();
	for (var i = 0 ; i < items.length; i++) {
		var item = items[i];
        var copyItem = item.duplicate(item,ElementPlacement.PLACEAFTER);
		item.transform(matrixL,true,true,true,true,1);
                        
        var matrixMove = app.getTranslationMatrix(item.width,0);
        var matrixFull = app.concatenateMatrix(matrixR,matrixMove);
      
        copyItem.transform(matrixFull,true,true,true,true,1);
		}
    }

function buildMsg (buttonName, angle, specifier) {
    var bt = new BridgeTalk;
    bt.target = specifier;		
    var params = {buttonName:buttonName, angle:angle};
    var msg = processClick + '\rprocessClick(' + params.toSource() + ');';

    bt.body = msg;
    bt.send();	
}

function main() {
	new IsometryLightInterface().run(BridgeTalk.appSpecifier);
	}

main();

